;# data.s	
;# checks all the data transfer instructions
foo:
	XCH	
	XCHN
	MOVE NUL, #01h
	MOVE NUL, AP
	MOVE NUL, APC
	MOVE NUL, PSF
	MOVE NUL, IC
	MOVE NUL, IMR
	MOVE NUL, SC
	MOVE NUL, IIR
	MOVE NUL, CKCN
	MOVE NUL, WDCN
	MOVE NUL, A[0]		;Just Check two boundary conditions
	MOVE NUL, A[15]   
	MOVE NUL, ACC
	MOVE NUL, A[AP]
	MOVE NUL, IP
	MOVE NUL, @SP--
	MOVE NUL, SP
	MOVE NUL, IV
	MOVE NUL, LC[0]
	MOVE NUL, LC[1]
	MOVE NUL, @BP[OFFS++]
	MOVE NUL, @BP[OFFS--]
	MOVE NUL, OFFS
	MOVE NUL, DPC
	MOVE NUL, GR
	MOVE NUL, GRL
	MOVE NUL, BP
	MOVE NUL, GRS
	MOVE NUL, GRH
	MOVE NUL, GRXL
	MOVE NUL, FP
	MOVE NUL, @DP[0]	
	MOVE NUL, @DP[1]	
	MOVE NUL, @DP[0]++		
	MOVE NUL, @DP[1]++
	MOVE NUL, @DP[0]--
	MOVE NUL, @DP[1]--
	MOVE AP, #01h
	MOVE AP, APC
	MOVE AP, PSF
	MOVE AP, IC
	MOVE AP, IMR
	MOVE AP, SC
	MOVE AP, IIR
	MOVE AP, CKCN
	MOVE AP, WDCN
	MOVE AP, A[0]		;Just Check two boundary conditions
	MOVE AP, A[15]   
	MOVE AP, ACC
	MOVE AP, A[AP]
	MOVE AP, IP
	MOVE AP, @SP--
	MOVE AP, SP
	MOVE AP, IV
	MOVE AP, LC[0]
	MOVE AP, LC[1]
	MOVE AP, @BP[OFFS++]
	MOVE AP, @BP[OFFS--]
	MOVE AP, OFFS
	MOVE AP, DPC
	MOVE AP, GR
	MOVE AP, GRL
	MOVE AP, BP
	MOVE AP, GRS
	MOVE AP, GRH
	MOVE AP, GRXL
	MOVE AP, FP
	MOVE AP, @DP[0]	
	MOVE AP, @DP[1]	
	MOVE AP, @DP[0]++		
	MOVE AP, @DP[1]++
	MOVE AP, @DP[0]--
	MOVE AP, @DP[1]--
	MOVE APC, #01h
	MOVE APC, AP
	MOVE APC, PSF
	MOVE APC, IC
	MOVE APC, IMR
	MOVE APC, SC
	MOVE APC, IIR
	MOVE APC, CKCN
	MOVE APC, WDCN
	MOVE APC, A[0]		;Just Check two boundary conditions
	MOVE APC, A[15]   
	MOVE APC, ACC
	MOVE APC, A[AP]
	MOVE APC, IP
	MOVE APC, @SP--
	MOVE APC, SP
	MOVE APC, IV
	MOVE APC, LC[0]
	MOVE APC, LC[1]
	MOVE APC, @BP[OFFS++]
	MOVE APC, @BP[OFFS--]
	MOVE APC, OFFS
	MOVE APC, DPC
	MOVE APC, GR
	MOVE APC, GRL
	MOVE APC, BP
	MOVE APC, GRS
	MOVE APC, GRH
	MOVE APC, GRXL
	MOVE APC, FP
	MOVE APC, @DP[0]	
	MOVE APC, @DP[1]	
	MOVE APC, @DP[0]++		
	MOVE APC, @DP[1]++
	MOVE APC, @DP[0]--
	MOVE APC, @DP[1]--
