#readelf: -srx1
There are no relocations in this file\.

Symbol table '\.symtab' contains . entries:
   Num:    Value          Size Type    Bind   Vis      Ndx Name
#...
     .: 8000000000000000     0 NOTYPE  GLOBAL DEFAULT  ABS __\.MMIX\.start\.\.text

Hex dump of section '\.text':
  0x0+ f4000000   .*
