
Symbol table '.symtab' contains .. entries:
   Num:    Value  Size Type    Bind   Vis      Ndx Name
#...
    ..: 0[0-9a-f]+[ 	]+7 FUNC    LOCAL  DEFAULT    . _func
#...
    ..: 0[0-9a-f]+[ 	]+0 NOTYPE  LOCAL  DEFAULT    . A
    ..: 0[0-9a-f]+[ 	]+7 FUNC    GLOBAL DEFAULT    . _func2
#...
    ..: 0[0-9a-f]+[ 	]+0 NOTYPE  GLOBAL DEFAULT    . BOTTOM
#...
